@@ -1,9 +1,8 @@
-from dataclasses import asdict, dataclass
+from dataclasses import dataclass
 from typing import Any
 
 from apischema.utils import (
-    GeneratorValue,
-    NO_DEFAULT,
+    Nil,
     as_dict,
     to_camel_case,
     to_hashable,
@@ -17,25 +16,14 @@ def test_to_hashable():
     assert hash(hashable1) == hash(hashable2)
 
 
-def test_generator_value():
-    def generator():
-        yield 0
-        return 1
-
-    value = GeneratorValue(generator())
-    assert list(value) == [0]
-    assert value.value == 1
-
-
 def test_to_camel_case():
     assert to_camel_case("min_length") == "minLength"
 
 
 @dataclass
 class Data:
-    default: Any = NO_DEFAULT
+    default: Any = Nil
 
 
 def test_as_dict():
-    assert as_dict(Data()) == {"default": NO_DEFAULT}
-    assert asdict(Data()) != {"default": NO_DEFAULT}  # because of deepcopy ...
+    assert as_dict(Data()) == {"default": Nil}
